/**
 * This firmware manages PWM generation and
 * dynamic voltage adjustment for the ATTiny45/85
 * of the XDC clock generator.
 *
 * Author: Bernhard Wrndl-Aichriedler
 * E-Mail: bwa@berniwa.com
 * 4HNc 2007/08
 * HTL-Salzburg 
 *
 * Last changes: 18.03.2010
 *
 * @brief XDC Firmware
 * @author Bernhard Wrndl-Aichriedler (bwa@berniwa.com)
 * @license GNU GPL
 */

#define TOGGLE_PB4(){if(PORTB & (1 << PB4)){PORTB &= ~(1 << PB4);}	else{PORTB |= (1 << PB4);}}
#define TOGGLE_PB3(){if(PORTB & (1 << PB3)){PORTB &= ~(1 << PB3);}	else{PORTB |= (1 << PB3);}}


/* Include interrupt header, to be able to use sei() to enable interrupts */
#include <avr/interrupt.h>

/* Include ADC header to be able to use ATtinys ADC functionality */
#include "xdc.h"

#include "twiSlave.h"

#define XDC_SLAVE_ADDRESS 0x30

/**
 * Register mapping
 * 
 * Register | Description
 * -----------------------------------------------------------
 * 0..1 	| Reference value for voltage limit, 
 *			| values from 0..1023 represent the voltages from 
 *			| 0V to the supply voltage of the IC
 * 2..3		| Refernece value for current limit, 
 *			| values from 0..1023 represent the voltages from 
 *			| 0V to the supply voltage of the IC
 * 4		| Hysteresis on voltage reference
 * 5		| Hysteresis on current reference
 * 6		| XDC Enable
 */


/* Main programm */
int main(void)
{
	/* Init XDC */
	xdc_init();	

	/* Prepare TWI Slave and registers */
	twiSlaveInit(XDC_SLAVE_ADDRESS, xdc_loadRegisters);
	xdc_prefillRegisters();

	/* Global interrupt enable */
	sei();

	while(1)
	{
		xdc_adjust();
	}
}
