#include "twiSlave.h"

/* Include TWI Slave driver provided by Atmel */
#include "USI_TWI_Slave.h"

#include "xdc.h"

#include <avr/io.h>
#include <avr/interrupt.h>


#define TWI_SLAVE_REGISTER_COUNT 10
#define TWI_SLAVE_MODE_MASTER_TO_SLAVE 1
#define TWI_SLAVE_MODE_SLAVE_TO_MASTER 0

/* TWI Slave configuration */
static unsigned char twiSlaveRegisterPointer = 0;
static unsigned char twiSlaveRegisters[TWI_SLAVE_REGISTER_COUNT];
static void (*twiSlaveUpdateCallback)() = 0;

void twiSlaveInit(unsigned char slaveAddress, void (*updateCallback)())
{
	twiSlaveUpdateCallback = updateCallback;

	unsigned char i;
	for(i = 0; i < TWI_SLAVE_REGISTER_COUNT; i++)
	{
		twiSlaveRegisters[i] = 0;
	}

	USI_TWI_Slave_Initialise( slaveAddress , twiSlaveStartCallback);
		
}

unsigned char twiSlaveRegisterRead(unsigned char reg)
{
	return twiSlaveRegisters[reg];
}
void twiSlaveRegisterWrite(unsigned char reg, unsigned char data)
{
	twiSlaveRegisters[reg] = data;	
}

void twiSlaveStartCallback(unsigned char mode)
{
	if(mode == TWI_SLAVE_MODE_SLAVE_TO_MASTER)
	{
		twiSlaveTimerClear();
		USI_TWI_Transmit_Byte(twiSlaveRegisters[twiSlaveRegisterPointer]);
		twiSlaveRegisterPointer = (twiSlaveRegisterPointer+1)%TWI_SLAVE_REGISTER_COUNT;
	}
	else if(mode == TWI_SLAVE_MODE_MASTER_TO_SLAVE)
	{
		twiSlaveReadComplete();
		twiSlaveTimerInit();
		USI_TWI_Reset_Receive_Buffer();
	}
}

ISR(SIG_OVERFLOW0)
{
	if(USISR & (1 << USIPF))
	{
		/* Clear stop condition flag */ 
		USISR |= (1 << USIPF);

		twiSlaveReadComplete();
	}
}

void twiSlaveReadComplete()
{
	if(USI_TWI_Data_In_Receive_Buffer())
	{
		twiSlaveRegisterPointer = USI_TWI_Receive_Byte() % TWI_SLAVE_REGISTER_COUNT;
	}

	while(USI_TWI_Data_In_Receive_Buffer())
	{
		twiSlaveRegisters[twiSlaveRegisterPointer] = USI_TWI_Receive_Byte();
		twiSlaveRegisterPointer = (twiSlaveRegisterPointer+1)%TWI_SLAVE_REGISTER_COUNT;
	}

	twiSlaveTimerClear();

	twiSlaveUpdateCallback();	
}

void twiSlaveTimerInit()
{
	TCCR0A = 0;
	//TCCR1A = (1 << WGM1); //CTC Mode

	TCCR0B = 0; //Reset timer register
	TCCR0B |= 0b00000010; //Set prescaler to 64
	TCCR0B |= (1 << PWM1A) | (1 << COM1A1); //Set PWM mode
	TIMSK |= (1 << TOIE0);	
}

void twiSlaveTimerClear()
{
	TCCR0B = 0; //Prescaler 0 -> Timer stopped	
}


